import { Component, Input, Output, EventEmitter, ViewChild } from "@angular/core";
var AudioComponent = (function () {
    function AudioComponent() {
        this.window = window;
        /**
         * @Input -> custom properties.
         *
         */
        /** Programmatically buttons. */
        this.playButton = false;
        this.pauseButton = false;
        this.selectableButton = false;
        this.muteButton = false;
        /** Array of audio tracks.*/
        this.src = [];
        /** Display or not the controls, default: true */
        this.controls = true;
        /** Set autoplay status, default true. */
        this.autoplay = true;
        /** Set loop status, default false. */
        this.loop = false;
        /** Set the volume, default: 1 (max). */
        this.volume = 1;
        /** Set the start index of the playlist. */
        this.startPosition = 0;
        /** Number in s, in order to start the transition, default: 5s */
        this.transition = 5;
        /** Interval in order to set the audio transition, in ms, default: 500ms. */
        this.intervalTransition = 500;
        /** Define if transition, default: false. */
        this.transitionEnd = true;
        /** Define the preload status, default metadata. */
        this.transitionStart = false;
        /** Define the preload status, default metadata. */
        this.preload = "metadata";
        /** Define the mute status, default false. */
        this.muted = false;
        /**
         * Custom events who could be intercepted.
         * @type {EventEmitter}
         */
        /** Emit the playlist. */
        this.playlist = new EventEmitter();
        /** Emit informations on the current video. */
        this.current = new EventEmitter();
        /** Emit the progress status of audio dowloading. */
        this.progress = new EventEmitter();
        /** Emit downloading status of track. */
        this.downloading = new EventEmitter();
    }
    AudioComponent.prototype.ngOnInit = function () {
        /** Init player with the first occurrence of src's array. */
        if (this.src.length) {
            this.currentTrack = this.src[this.startPosition];
        }
    };
    AudioComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.transitionEnd) {
            this.player.nativeElement.addEventListener("play", function () {
                return _this.audioTransition(_this.player.nativeElement.duration, _this.player.nativeElement.currentTime);
            });
        }
        this.player.nativeElement.addEventListener("ended", function () {
            _this.player.nativeElement.volume = _this.volume;
            /** Increment array position in order to get next audio track. */
            _this.startPosition += 1;
            /** If loop is true && startPosition is at last index then reset the playlist. */
            if (_this.startPosition >= _this.src.length && _this.loop)
                _this.startPosition = 0;
            /** Else stop the playlist. */
            if (_this.startPosition >= _this.src.length && !_this.loop)
                return;
            /** Set new src track */
            _this.player.nativeElement.src = _this.src[_this.startPosition];
            /** If onChangeTrack is set, then emit the new track. */
        });
        this.player.nativeElement.addEventListener("loadstart", function () {
            _this.emitCurrentTrack();
            if (_this.transitionStart)
                _this.audioStartTransition(_this.intervalTransition);
        });
        this.player.nativeElement.addEventListener("pause", function () {
            /** Reset Timeout && Interval. */
            _this.window.clearTimeout(_this.timeout);
            _this.window.clearInterval(_this.interval);
        });
        this.player.nativeElement.addEventListener("progress", function (data) {
            return _this.downloading.emit(true);
        });
    };
    /** Set programmatically audio controls. */
    AudioComponent.prototype.play = function () {
        this.player.nativeElement.play();
    };
    AudioComponent.prototype.pause = function () {
        this.player.nativeElement.pause();
    };
    AudioComponent.prototype.muteVideo = function () {
        this.player.nativeElement.muted = !this.player.nativeElement.muted;
    };
    AudioComponent.prototype.previousTrack = function () {
        /** If first track, then do nothing. */
        if (this.src.indexOf(this.player.nativeElement.src) <= 0) {
            return;
        }
        /** Else go back to previous element in track's array. */
        this.player.nativeElement.src = this.src[this.src.indexOf(this.player.nativeElement.src) - 1];
    };
    AudioComponent.prototype.nextTrack = function () {
        /** If last track, then do nothing. */
        if (this.src.indexOf(this.player.nativeElement.src) >=
            this.src.length - 1) {
            return;
        }
        /** Else, go to the next element in track's array. */
        this.player.nativeElement.src = this.src[this.src.indexOf(this.player.nativeElement.src) + 1];
    };
    /** Audio Transitions */
    /** Set transition audio. */
    AudioComponent.prototype.audioTransition = function (trackDuration, timeElapsed) {
        if (timeElapsed === void 0) { timeElapsed = 0; }
        /** Clear setInterval if defined. */
        this.window.clearInterval(this.interval);
        /** Check the currentTime elapsed, then set transition if defined. */
        this.timeout = this.setTimeoutDelay(trackDuration, timeElapsed);
    };
    AudioComponent.prototype.audioStartTransition = function (interval) {
        /** Start the transition. */
        this.startTransition = this.setIncrementInterval(interval);
    };
    AudioComponent.prototype.setTimeoutDelay = function (trackDuration, timeElapsed) {
        var _this = this;
        /** Timeout who correspond to the remaining time of audio player without the transition's time ( by default 5s before the end). */
        return setTimeout(function () {
            _this.interval = _this.setDecrementInterval(_this.intervalTransition);
        }, (trackDuration - timeElapsed) * 1000 - this.transition * 1000);
    };
    AudioComponent.prototype.setIncrementInterval = function (interval) {
        var _this = this;
        return setInterval(function () {
            /** Define the new player's volume. Increment by step of 10%.*/
            _this.player.nativeElement.volume +=
                (_this.player.nativeElement.volume * 10) / 100;
            /** Security area in order to avoid error. If the player's volume is around 90%, then stop incrment, set the volume to 1. */
            if (_this.player.nativeElement.volume >= 0.9) {
                _this.player.nativeElement.volume = 1;
                _this.window.clearInterval(_this.startTransition);
            }
        }, interval);
    };
    AudioComponent.prototype.setDecrementInterval = function (interval) {
        var _this = this;
        return setInterval(function () {
            /** Decrement the volume by step of 10%. */
            _this.player.nativeElement.volume -=
                (_this.player.nativeElement.volume * 10) / 100;
        }, interval);
    };
    /**
     * Emitters
     */
    AudioComponent.prototype.emitPlayList = function () {
        this.playlist.emit(this.src);
    };
    AudioComponent.prototype.emitCurrentTrack = function () {
        /**
         * Return an object who will contain: Url of the track, duration, textTrack, volume)
         */
        this.current.emit({
            src: this.player.nativeElement.currentSrc,
            textTracks: this.player.nativeElement.textTracks,
            volume: this.player.nativeElement.volume
        });
    };
    return AudioComponent;
}());
export { AudioComponent };
AudioComponent.decorators = [
    { type: Component, args: [{
                selector: "audio-component",
                template: "\n    <audio\n      #audioplayer\n      [src]=\"currentTrack\"\n      [autoplay]=\"autoplay\"\n      [controls]=\"controls\"\n      [volume]=\"volume\"\n      [loop]=\"loop\"\n      [preload]=\"preload\"\n      [muted]=\"muted\"\n    ></audio>\n    <button type=\"button\" (click)=\"play()\" *ngIf=\"playButton\">Play</button>\n    <button type=\"button\" (click)=\"pause()\" *ngIf=\"pauseButton\">Pause</button>\n    <button type=\"button\" (click)=\"previousTrack()\" *ngIf=\"selectableButton\">\n      Previous\n    </button>\n    <button type=\"button\" (click)=\"nextTrack()\" *ngIf=\"selectableButton\">\n      Next\n    </button>\n    <button type=\"button\" (click)=\"muteVideo()\" *ngIf=\"muteButton\">Mute</button>\n  "
            },] },
];
/** @nocollapse */
AudioComponent.ctorParameters = function () { return []; };
AudioComponent.propDecorators = {
    'playButton': [{ type: Input },],
    'pauseButton': [{ type: Input },],
    'selectableButton': [{ type: Input },],
    'muteButton': [{ type: Input },],
    'src': [{ type: Input },],
    'controls': [{ type: Input },],
    'autoplay': [{ type: Input },],
    'loop': [{ type: Input },],
    'volume': [{ type: Input },],
    'startPosition': [{ type: Input },],
    'transition': [{ type: Input },],
    'intervalTransition': [{ type: Input },],
    'transitionEnd': [{ type: Input },],
    'transitionStart': [{ type: Input },],
    'preload': [{ type: Input },],
    'muted': [{ type: Input },],
    'playlist': [{ type: Output },],
    'current': [{ type: Output },],
    'progress': [{ type: Output },],
    'downloading': [{ type: Output },],
    'player': [{ type: ViewChild, args: ["audioplayer",] },],
};
